@@ -123,7 +123,7 @@ class Configuration(MutableMapping):
     __str__ = __repr__
 
     @classmethod
-    def from_file(cls, filename, ctx=None):
+    def from_file(cls, filename, ctx=None, constructors=None):
         """ Construct :class:`.Configuration` object by reading and parsing file
         ``filename``.
 
@@ -131,9 +131,12 @@ class Configuration(MutableMapping):
             filename to parse config from
         :param ctx:
             mapping object used for value interpolation
+        :param constructors:
+            mapping of names to constructor for custom objects in YAML. Look at
+            `_timedelta_constructor` and `_re_constructor` for examples.
         """
         with open(filename, "r") as f:
-            cfg = cls(load(f.read()), ctx=ctx)
+            cfg = cls(load(f.read(), constructors), ctx=ctx)
         if "extends" in cfg:
             supcfg_path = path.join(path.dirname(filename), cfg.pop("extends"))
             supcfg = cls.from_file(supcfg_path)
@@ -141,15 +144,18 @@ class Configuration(MutableMapping):
         return cfg
 
     @classmethod
-    def from_string(cls, string, ctx=None):
+    def from_string(cls, string, ctx=None, constructors=None):
         """ Construct :class:`.Configuration` from ``string``.
 
         :param string:
             string to parse config from
         :param ctx:
             mapping object used for value interpolation
+        :param constructors:
+            mapping of names to constructor for custom objects in YAML. Look at
+            `_timedelta_constructor` and `_re_constructor` for examples.
         """
-        return cls(load(string), ctx=ctx)
+        return cls(load(string, constructors), ctx=ctx)
 
     @classmethod
     def from_dict(cls, d, ctx=None):
@@ -177,7 +183,6 @@ def configure_logging(logcfg=None, disable_existing_loggers=True):
                 "handlers": {
                     "syslog": {
                         "class": "logging.handlers.SysLogHandler",
-                        "level": "INFO",
                         "formatter": "precise",
                     }
                 },
@@ -187,15 +192,7 @@ def configure_logging(logcfg=None, disable_existing_loggers=True):
                 }
             }
     else:
-        logcfg = {
-            "handlers": {
-                "console": {
-                    "class": "logging.StreamHandler",
-                    "level": "INFO",
-                    "formatter": "precise",
-                }
-            }
-        }
+        logcfg = {}
 
     if not "version" in logcfg:
         logcfg["version"] = 1
@@ -203,6 +200,8 @@ def configure_logging(logcfg=None, disable_existing_loggers=True):
     if not "disable_existing_loggers" in logcfg:
         logcfg["disable_existing_loggers"] = disable_existing_loggers
 
+    # formatters
+
     if not "formatters" in logcfg:
         logcfg["formatters"] = {}
 
@@ -216,24 +215,34 @@ def configure_logging(logcfg=None, disable_existing_loggers=True):
             "format": "%(asctime)s %(levelname)-8s %(name)-15s %(message)s",
         }
 
+    # handlers
+
     if not "root" in logcfg:
         logcfg["root"] = {
             "handlers": ["console"],
             "level": "NOTSET",
         }
 
-    if not "syslog" in logcfg.get("handlers", {}):
-        logcfg.setdefault("handlers", {})["syslog"] = {
-            "class": "logging.handlers.SysLogHandler"
+    if not "handlers" in logcfg:
+        logcfg["handlers"] = {}
+
+    if not "syslog" in logcfg["handlers"]:
+        logcfg["handlers"]["syslog"] = {
+            "class": "logging.handlers.SysLogHandler",
+            "formatter": "precise",
+            "level": "NOTSET",
         }
 
-    if not "console" in logcfg.get("handlers", {}):
-        logcfg.setdefault("handlers", {})["console"] = {
+    if not "console" in logcfg["handlers"]:
+        logcfg["handlers"]["console"] = {
             "class": "logging.StreamHandler",
-            "level": "INFO",
+            "formatter": "precise",
+            "level": "NOTSET",
         }
 
     from logging.config import dictConfig
+    from pprint import pprint
+    pprint(logcfg)
     dictConfig(logcfg)
 
 def _timedelta_contructor(loader, node):
@@ -266,10 +275,13 @@ def _re_constructor(loader, node):
     item = loader.construct_scalar(node)
     return re_compile(item)
 
-def load(stream):
+def load(stream, constructors=None):
     loader = Loader(stream)
     loader.add_constructor("!timedelta", _timedelta_contructor)
     loader.add_constructor("!re", _re_constructor)
+    if constructors:
+        for name, constructor in constructors.items():
+            loader.add_constructor("!" + name, constructor)
     try:
         return loader.get_single_data()
     finally:
